-------------------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_signed.ALL;
USE work.ImPack.ALL;

ENTITY ImOutE IS
  PORT (
    Reset  : IN  std_logic;             -- Reset, low-aktiv
    Clk50  : IN  std_logic;             -- Takt FPGA-Board
    R      : OUT std_logic_vector(7 DOWNTO 0);       -- Pixel rot
    G      : OUT std_logic_vector(7 DOWNTO 0);       -- Pixel gruen
    B      : OUT std_logic_vector(7 DOWNTO 0);       -- Pixel blau
    HSync  : OUT std_logic;             -- Sync horizontal fuer DAC
    VSync  : OUT std_logic;             -- Sync vertikal fuer DAC
    Clk25  : OUT std_logic;  -- Takt fuer DAC                                     
    ClkDiv : OUT std_logic_vector(CNDiv DOWNTO 0));  -- Ausgang Taktteiler 
END ImOutE;

ARCHITECTURE ImOutA OF ImOutE IS
	Component ImReadE IS
	PORT (
    Clk25 : IN  std_logic;                            -- clock vga
    Reset : IN  std_logic;                            -- Reset, low-active
    PixR  : IN  std_logic_vector(CWidth-1 DOWNTO 0);  -- ram data
    AddrR : OUT std_logic_vector(CDepth-1 DOWNTO 0);  -- ram address
    R     : OUT std_logic_vector(7 DOWNTO 0);         -- pixel R value
    G     : OUT std_logic_vector(7 DOWNTO 0);         -- pixel G value
    B     : OUT std_logic_vector(7 DOWNTO 0);         -- pixel B value
    HSync : OUT std_logic;                            -- horizontal sync
    VSync : OUT std_logic);
	end component;
	
	Component ClkDivE IS
	  PORT (
		 Reset  : IN    std_logic;           -- Reset, low-aktiv
		 Clk25  : IN    std_logic;           -- Takt FPGA-Board
		 ClkDiv : INOUT std_logic_vector(CNDiv DOWNTO 0));                              
	END component;
	
	component clockgen
    port (
      U1_CLKIN_IN        : in  std_logic;
      U1_RST_IN          : in  std_logic;
      clk12              : out std_logic;
      clk25              : out std_logic;
      U1_CLKDV_OUT       : out std_logic;
      U1_CLKFX_OUT       : out std_logic;
      U1_CLKIN_IBUFG_OUT : out std_logic;
      U1_CLK0_OUT        : out std_logic;
      U1_STATUS_OUT      : out std_logic_vector (7 downto 0);
      U2_CLKFX_OUT       : out std_logic;
      U2_CLK0_OUT        : out std_logic;
      U2_LOCKED_OUT      : out std_logic;
      U2_STATUS_OUT      : out std_logic_vector (7 downto 0));
  end component;
  
  component Ramqvga
    port (
      clka  : in  std_logic;
      wea   : in  std_logic_vector(0 downto 0);
      addra : in  std_logic_vector(CDepth-1 downto 0);
      dina  : in  std_logic_vector(11 downto 0);
      clkb  : in  std_logic;
      addrb : in  std_logic_vector(CDepth-1 downto 0);
      doutb : out std_logic_vector(11 downto 0));
  end component;
  
	signal CLK25_S : std_logic;
	signal AddrR_S : std_logic_vector(CDepth-1 DOWNTO 0);
	signal doutb_S : std_logic_vector(11 downto 0);
	signal not_Reset_S : std_logic;
	signal ClkDiv_S : std_logic_vector(CNDiv DOWNTO 0);
	
BEGIN

	not_Reset_S <= not Reset;
	ClkDiv <= ClkDiv_S;
	Clk25 <= Clk25_S;

	ramm1 : Ramqvga
	port map(
		clka  => '0' ,
      wea   => (others => '1'), 
      addra => (others => '0'),
      dina  => (others => '0'),
      clkb  => CLK25_S,
      addrb => AddrR_S,
      doutb => doutb_S
		);	
		
	clockgen1: clockgen
    port map (
      U1_CLKIN_IN        => Clk50,
      U1_RST_IN          => not_Reset_S,
      clk12              => open,
      clk25              => CLK25_S,
      U1_CLKDV_OUT       => open,
      U1_CLKFX_OUT       => open,
      U1_CLKIN_IBUFG_OUT => open,
      U1_CLK0_OUT        => open,
      U1_STATUS_OUT      => open,
      U2_CLKFX_OUT       => open,
      U2_CLK0_OUT        => open,
      U2_LOCKED_OUT      => open,
      U2_STATUS_OUT      => open
		);

	ClkDiv1 : ClkDivE 
	PORT MAP(
		 Reset  => Reset,
		 Clk25  => Clk25_S,
		 ClkDiv => ClkDiv_S
		 );                              

	ImRead: ImReadE
	PORT MAP(
    Clk25 => CLK25_S,                           
    Reset => Reset,                           
    PixR  => doutb_S,
    AddrR => AddrR_S,
    R     => R,         
    G     => G,       
    B     => B,
    HSync => HSync,
    VSync => VSync
	 );
	

END ImOutA;
